@@ -24,6 +24,7 @@ enum
 	COL_SELECTED,
 	COL_STATUS,
 	COL_PATH,
+	COL_DIFF,
 	COL_TYPE
 };
 
@@ -60,12 +61,14 @@ struct _GitStatusPanePriv
 
 	/* Iters for the two sections: Changes to be committed and Changed but not
 	 * updated. Status items will be children of these two iters. */
-	GtkTreeIter commit_iter;
-	GtkTreeIter not_updated_iter;
+	GtkTreePath *commit_section;
+	GtkTreePath *not_updated_section;
 
 	/* Hash tables that show which items are selected in each section */
 	GHashTable *selected_commit_items;
 	GHashTable *selected_not_updated_items;
+
+	gboolean show_diff;
 };
 
 G_DEFINE_TYPE (GitStatusPane, git_status_pane, GIT_TYPE_PANE);
@@ -84,7 +87,7 @@ selected_renderer_data_func (GtkTreeViewColumn *tree_column,
 	 * Changeed but not updated. */
 	gtk_cell_renderer_set_visible (renderer, 
 	                               gtk_tree_store_iter_depth (GTK_TREE_STORE (model), 
-	                                                          iter) > 0);
+	                                                          iter) == 1);
 
 	gtk_tree_model_get (model, iter, COL_SELECTED, &selected, -1);
 
@@ -105,7 +108,7 @@ status_icon_renderer_data_func (GtkTreeViewColumn *tree_column,
 	/* Don't show this renderer on placeholders */
 	gtk_cell_renderer_set_visible (renderer, 
 	                               gtk_tree_store_iter_depth (GTK_TREE_STORE (model), 
-	                                                          iter) > 0);
+	                                                          iter) == 1);
 
 	gtk_tree_model_get (model, iter, COL_STATUS, &status, -1);
 
@@ -167,7 +170,7 @@ status_name_renderer_data_func (GtkTreeViewColumn *tree_column,
 	/* Don't show this renderer on placeholders */
 	gtk_cell_renderer_set_visible (renderer, 
 	                               gtk_tree_store_iter_depth (GTK_TREE_STORE (model), 
-	                                                          iter) > 0);
+	                                                          iter) == 1);
 
 	switch (status)
 	{
@@ -220,6 +223,11 @@ path_renderer_data_func (GtkTreeViewColumn *tree_column,
 
 	gtk_tree_model_get (model, iter, COL_PATH, &path, -1);
 
+	/* Don't show this column on diffs */
+	gtk_cell_renderer_set_visible (renderer,
+	                               gtk_tree_store_iter_depth (GTK_TREE_STORE (model),
+	                                                          iter)  != 2);
+
 	/* Use the path column to show placeholders as well */
 	if (gtk_tree_store_iter_depth (GTK_TREE_STORE (model), iter) == 0)
 	{
@@ -301,14 +309,60 @@ on_selected_renderer_toggled (GtkCellRendererToggle *renderer, gchar *tree_path,
 	g_free (path);
 }
 
+static void
+on_diff_command_finished (AnjutaCommand *command, guint return_code, 
+                          gpointer data)
+{
+	GtkTreeModel *status_model;
+	GtkTreePath *parent_path;
+	GtkTreeIter parent_iter;
+	GtkTreeIter iter;
+	GString *string;
+	GQueue *output;
+	gchar *output_line;
+	GtkTreeView *status_view;
+
+	if (return_code == 0)
+	{
+		status_model = g_object_get_data (G_OBJECT (command), "model");
+		parent_path = g_object_get_data (G_OBJECT (command), "parent-path");
+		gtk_tree_model_get_iter (status_model, &parent_iter, parent_path);
+		string = g_string_new ("");
+		output = git_raw_output_command_get_output (GIT_RAW_OUTPUT_COMMAND (command));
+
+		while (g_queue_peek_head (output))
+		{
+			output_line = g_queue_pop_head (output);
+			g_string_append (string, output_line);
+
+			g_free (output_line);
+		}
+
+		gtk_tree_store_append (GTK_TREE_STORE (status_model), &iter, &parent_iter);
+		gtk_tree_store_set (GTK_TREE_STORE (status_model), &iter, COL_DIFF, string->str, -1);
+
+		if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (command), "show-diff")))
+		{
+			status_view = g_object_get_data (G_OBJECT (command), "status-view");
+			gtk_tree_view_expand_row (status_view, parent_path, FALSE);
+		}
+
+		g_string_free (string, TRUE);
+	}
+}
+
 static void
 add_status_items (GQueue *output, GtkTreeStore *status_model, 
-                  GtkTreeIter *parent_iter, StatusType type)
+                  GtkTreeView *status_view, GtkTreePath *parent, 
+                  StatusType type, const gchar *working_directory, 
+                  gboolean show_diff)
 {
 	GitStatus *status_object;
 	AnjutaVcsStatus status;
 	gchar *path;
+	GtkTreeIter parent_iter;
 	GtkTreeIter iter;
+	GitDiffCommand *diff_command;
 
 	while (g_queue_peek_head (output))
 	{
@@ -316,7 +370,9 @@ add_status_items (GQueue *output, GtkTreeStore *status_model,
 		status = git_status_get_vcs_status (status_object);
 		path = git_status_get_path (status_object);
 
-		gtk_tree_store_append (status_model, &iter, parent_iter);
+		gtk_tree_model_get_iter (GTK_TREE_MODEL (status_model), &parent_iter, 
+		                         parent);
+		gtk_tree_store_append (status_model, &iter, &parent_iter);
 		gtk_tree_store_set (status_model, &iter,
 		                    COL_SELECTED, FALSE,
 		                    COL_STATUS, status,
@@ -324,8 +380,30 @@ add_status_items (GQueue *output, GtkTreeStore *status_model,
 		                    COL_TYPE, type,
 		                    -1);
 
+		diff_command = git_diff_command_new (working_directory, path,
+		                                     type == STATUS_TYPE_NOT_UPDATED ? GIT_DIFF_WORKING_TREE : GIT_DIFF_INDEX);
+		
+		g_signal_connect (G_OBJECT (diff_command), "command-finished",
+		                  G_CALLBACK (on_diff_command_finished),
+		                  NULL);
+
+		g_signal_connect (G_OBJECT (diff_command), "command-finished",
+		                  G_CALLBACK (g_object_unref),
+		                  NULL);
+
+		g_object_set_data_full (G_OBJECT (diff_command), "parent-path", 
+		                        gtk_tree_model_get_path (GTK_TREE_MODEL (status_model), 
+		                                                 &iter),
+		                        (GDestroyNotify) gtk_tree_path_free);
+		g_object_set_data (G_OBJECT (diff_command), "model", status_model);
+		g_object_set_data (G_OBJECT (diff_command), "status-view", status_view);
+		g_object_set_data (G_OBJECT (diff_command), "show-diff", 
+		                   GINT_TO_POINTER (show_diff));
+		
 		g_free (path);
 		g_object_unref (status_object);
+
+		anjuta_command_start (ANJUTA_COMMAND (diff_command));
 	}
 }
 
@@ -334,15 +412,23 @@ on_commit_status_data_arrived (AnjutaCommand *command,
                                GitStatusPane *self)
 {
 	GtkTreeStore *status_model;
+	GtkTreeView *status_view;
 	GQueue *output;
+	gchar *working_directory;
 
 	status_model = GTK_TREE_STORE (gtk_builder_get_object (self->priv->builder,
 	                                                       "status_model"));
+	status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
+	                                                     "status_view"));
 	output = git_status_command_get_status_queue (GIT_STATUS_COMMAND (command));
+	g_object_get (G_OBJECT (command), "working-directory", &working_directory, 
+	              NULL);
 
-	add_status_items (output, status_model, &(self->priv->commit_iter),
-	                  STATUS_TYPE_COMMIT);
+	add_status_items (output, status_model, status_view, 
+	                  self->priv->commit_section, STATUS_TYPE_COMMIT,
+	                  working_directory, self->priv->show_diff);
 
+	g_free (working_directory);
 }
 
 static void
@@ -350,44 +436,60 @@ on_not_updated_status_data_arrived (AnjutaCommand *command,
                                     GitStatusPane *self)
 {
 	GtkTreeStore *status_model;
+	GtkTreeView *status_view;
 	GQueue *output;
+	gchar *working_directory;
 
 	status_model = GTK_TREE_STORE (gtk_builder_get_object (self->priv->builder,
 	                                                       "status_model"));
+	status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
+	                                                     "status_view"));
 	output = git_status_command_get_status_queue (GIT_STATUS_COMMAND (command));
+	g_object_get (G_OBJECT (command), "working-directory", &working_directory, 
+	              NULL);
 
-	add_status_items (output, status_model, &(self->priv->not_updated_iter),
-	                  STATUS_TYPE_NOT_UPDATED);
+	add_status_items (output, status_model, status_view, 
+	                  self->priv->not_updated_section, STATUS_TYPE_NOT_UPDATED, 
+	                  working_directory, self->priv->show_diff);
+
+	g_free (working_directory);
 }
 
 static void
 git_status_pane_clear (GitStatusPane *self)
 {
 	GtkTreeStore *status_model;
+	GtkTreeIter iter;
 
 	status_model = GTK_TREE_STORE (gtk_builder_get_object (self->priv->builder,	
 	                                                       "status_model"));
 
 	/* Clear any existing model data and create the placeholders */
 	gtk_tree_store_clear (status_model);
+
+	gtk_tree_path_free (self->priv->commit_section);
+	gtk_tree_path_free (self->priv->not_updated_section);
 	
-	gtk_tree_store_append (status_model, &(self->priv->commit_iter), NULL);
-	gtk_tree_store_set (status_model, &(self->priv->commit_iter), 
+	gtk_tree_store_append (status_model, &iter, NULL);
+	gtk_tree_store_set (status_model, &iter, 
 	                    COL_PATH, _("Changes to be committed"), 
 	                    COL_SELECTED, FALSE,
 	                    COL_STATUS, ANJUTA_VCS_STATUS_NONE,
 	                    COL_TYPE, STATUS_TYPE_NONE,
 	                    -1);
+	
+	self->priv->commit_section = gtk_tree_model_get_path (GTK_TREE_MODEL (status_model), &iter);
 
-	gtk_tree_store_append (status_model, &(self->priv->not_updated_iter), 
-	                       NULL);
-	gtk_tree_store_set (status_model, &(self->priv->not_updated_iter), 
+	gtk_tree_store_append (status_model, &iter, NULL);
+	gtk_tree_store_set (status_model, &iter, 
 	                    COL_PATH, _("Changed but not updated"), 
 	                    COL_SELECTED, FALSE, 
 	                    COL_STATUS, ANJUTA_VCS_STATUS_NONE,
 	                    COL_TYPE, STATUS_TYPE_NONE,
 	                    -1);
 
+	self->priv->not_updated_section = gtk_tree_model_get_path (GTK_TREE_MODEL (status_model), &iter);
+
 	g_hash_table_remove_all (self->priv->selected_commit_items);
 	g_hash_table_remove_all (self->priv->selected_not_updated_items);
 }
@@ -398,7 +500,8 @@ git_status_pane_set_selected_column_state (GitStatusPane *self,
                                            gboolean state)
 {
 	GtkTreeModel *status_model;
-	GtkTreeIter *parent_iter;
+	GtkTreePath *section;
+	GtkTreeIter parent_iter;
 	gint i;
 	GtkTreeIter iter;
 	gchar *path;
@@ -406,22 +509,25 @@ git_status_pane_set_selected_column_state (GitStatusPane *self,
 
 	status_model = GTK_TREE_MODEL (gtk_builder_get_object (self->priv->builder,
 	                                                       "status_model"));
+
 	switch (type)
 	{
 		case STATUS_TYPE_COMMIT:
-			parent_iter = &(self->priv->commit_iter);
+			section = self->priv->commit_section;
 			break;
 		case STATUS_TYPE_NOT_UPDATED:
-			parent_iter = &(self->priv->not_updated_iter);
+			section = self->priv->not_updated_section;
 			break;
 		default:
 			return;
 			break;
 	}
+
+	gtk_tree_model_get_iter (status_model, &parent_iter, section);
 	
 	i = 0;
 
-	while (gtk_tree_model_iter_nth_child (status_model, &iter, parent_iter,
+	while (gtk_tree_model_iter_nth_child (status_model, &iter, &parent_iter,
 	                                      i++))
 	{
 		gtk_tree_store_set (GTK_TREE_STORE (status_model), &iter, 
@@ -544,24 +650,32 @@ on_status_view_button_press_event (GtkWidget *widget, GdkEvent *event,
 	AnjutaPlugin *plugin;
 	AnjutaUI *ui;
 	GtkTreeView *status_view;
-	GtkTreeSelection *selection;
+	GtkTreePath *path;
+	gboolean path_valid;
 	GtkTreeModel *status_model;
 	GtkTreeIter iter;
 	StatusType status_type;
 	GtkMenu *menu;
+	gboolean ret = FALSE;;
 
 	button_event = (GdkEventButton *) event;
+	status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
+	                                                     "status_view"));
+	status_model = GTK_TREE_MODEL (gtk_builder_get_object (self->priv->builder,
+	                                                       "status_model"));
+	path_valid = gtk_tree_view_get_path_at_pos (status_view, 
+	                                            button_event->x, button_event->y,
+	                                  			&path, NULL, NULL, NULL);
 	menu = NULL;
 	
+	
 	if (button_event->type == GDK_BUTTON_PRESS && button_event->button == 3)
 	{
 		plugin = anjuta_dock_pane_get_plugin (ANJUTA_DOCK_PANE (self));
 		ui = anjuta_shell_get_ui (plugin->shell, NULL);
-		status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
-		                                                     "status_view"));
-		selection = gtk_tree_view_get_selection (status_view);
+		
 
-		if (gtk_tree_selection_get_selected (selection, &status_model, &iter))
+		if (path_valid && gtk_tree_model_get_iter (status_model, &iter, path))
 		{
 			gtk_tree_model_get (status_model, &iter, COL_TYPE, &status_type, 
 			                    -1);
@@ -585,9 +699,69 @@ on_status_view_button_press_event (GtkWidget *widget, GdkEvent *event,
 		}
 	}
 
-	return FALSE;
+	if (path_valid)
+	{
+		/* Don't forward key events to diff columns */
+		ret = gtk_tree_path_get_depth (path) == 3;
+
+		gtk_tree_path_free (path);
+	}
+
+	return ret;
 }
 
+static gboolean
+on_status_view_row_selected (GtkTreeSelection *selection, 
+                             GtkTreeModel *model,
+                             GtkTreePath *path,
+                             gboolean path_currently_selected,
+                             gpointer user_data)
+{
+	return gtk_tree_path_get_depth (path) == 2;
+}
+
+static void 
+git_status_pane_expand_placeholders_full (GitStatusPane *self, 
+                                          gboolean open_all)
+{
+	GtkTreeView *status_view;
+
+	if (self->priv->commit_section && self->priv->not_updated_section)
+	{
+		status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
+		                                                     "status_view"));
+
+		gtk_tree_view_expand_row (status_view, self->priv->commit_section, 
+		                          open_all);
+		gtk_tree_view_expand_row (status_view, self->priv->not_updated_section, 
+		                          open_all);
+	}
+}
+
+static void
+git_status_pane_expand_placeholders (GitStatusPane *self)
+{
+	git_status_pane_expand_placeholders_full (self, FALSE);
+}
+
+static void
+on_status_diff_button_toggled (GtkToggleButton *button, GitStatusPane *self)
+{
+	GtkTreeView *status_view;
+
+	self->priv->show_diff = gtk_toggle_button_get_active (button);
+
+	if (!self->priv->show_diff)
+	{
+		status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
+		                                                     "status_view"));
+
+		gtk_tree_view_collapse_all (status_view);
+	}
+
+	git_status_pane_expand_placeholders_full (self, self->priv->show_diff);
+}                                 
+
 static void
 git_status_pane_init (GitStatusPane *self)
 {
@@ -601,6 +775,9 @@ git_status_pane_init (GitStatusPane *self)
 	GtkCellRenderer *status_icon_renderer;
 	GtkCellRenderer *status_name_renderer;
 	GtkCellRenderer *path_renderer;
+	GtkCellRenderer *diff_renderer;
+	GtkTreeSelection *selection;
+	GtkWidget *status_diff_button;
 	GtkWidget *refresh_button;
 	GtkWidget *select_all_button;
 	GtkWidget *clear_button;
@@ -636,6 +813,9 @@ git_status_pane_init (GitStatusPane *self)
 	                                                                  "status_name_renderer"));
 	path_renderer = GTK_CELL_RENDERER (gtk_builder_get_object (self->priv->builder,
 	                                                           "path_renderer"));
+	diff_renderer = anjuta_cell_renderer_diff_new ();
+	status_diff_button = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
+	                                                         "status_diff_button"));
 	refresh_button = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
 	                                                     "refresh_button"));
 	select_all_button = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
@@ -659,10 +839,24 @@ git_status_pane_init (GitStatusPane *self)
 	                                         (GtkTreeCellDataFunc) path_renderer_data_func,
 	                                         NULL, NULL);
 
+	gtk_tree_view_column_pack_start (status_column, diff_renderer, TRUE);
+	gtk_tree_view_column_add_attribute (status_column, diff_renderer, "diff", COL_DIFF);
+	
 	g_signal_connect (G_OBJECT (selected_renderer), "toggled",
 	                  G_CALLBACK (on_selected_renderer_toggled),
 	                  self);
 
+	g_signal_connect (G_OBJECT (status_diff_button), "toggled",
+	                  G_CALLBACK (on_status_diff_button_toggled),
+	                  self);
+
+	/* Don't allow the user to select any row that doesn't have a path,
+	 * such as the placeholders or diff rows */
+	selection = gtk_tree_view_get_selection (status_view);
+	gtk_tree_selection_set_select_function (selection, 
+	                                        on_status_view_row_selected,
+	                                        NULL, NULL);
+
 	g_signal_connect_swapped (G_OBJECT (refresh_button), "clicked",
 	                          G_CALLBACK (anjuta_dock_pane_refresh),
 	                          self);
@@ -704,6 +898,8 @@ git_status_pane_finalize (GObject *object)
 	self = GIT_STATUS_PANE (object);
 
 	g_object_unref (self->priv->builder);
+	gtk_tree_path_free (self->priv->commit_section);
+	gtk_tree_path_free (self->priv->not_updated_section);
 	g_hash_table_destroy (self->priv->selected_commit_items);
 	g_hash_table_destroy (self->priv->selected_not_updated_items);
 	g_free (self->priv);
@@ -748,23 +944,21 @@ AnjutaDockPane *
 git_status_pane_new (Git *plugin)
 {
 	GitStatusPane *self;
-	GtkTreeView *status_view;
+	GObject *status_diff_button;
 
 	self = g_object_new (GIT_TYPE_STATUS_PANE, "plugin", plugin, NULL);
-	status_view = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder,
-														 "status_view"));
+	status_diff_button = gtk_builder_get_object (self->priv->builder, 
+	                                             "status_diff_button");
 
 	g_signal_connect_swapped (G_OBJECT (plugin->commit_status_command), 
 	                          "command-started",
 	                          G_CALLBACK (git_status_pane_clear),
 	                          self);
 
-	/* Expand the placeholders so something is visible to the user after 
-	 * refreshing */
 	g_signal_connect_swapped (G_OBJECT (plugin->not_updated_status_command),
 	                          "command-finished",
-	                          G_CALLBACK (gtk_tree_view_expand_all),
-	                          status_view);
+	                          G_CALLBACK (git_status_pane_expand_placeholders),
+	                          self);
 
 	g_signal_connect (G_OBJECT (plugin->commit_status_command),
 	                  "data-arrived",
@@ -776,6 +970,9 @@ git_status_pane_new (Git *plugin)
 	                  G_CALLBACK (on_not_updated_status_data_arrived),
 	                  self);
 
+	g_settings_bind (plugin->settings, "show-status-diff",
+	                 status_diff_button, "active", G_SETTINGS_BIND_DEFAULT);
+
 	return ANJUTA_DOCK_PANE (self);
 }
 
